######################################
# Statistics for data science: R 
# 
# http://didawiki.di.unipi.it/doku.php/mds/sds/
######################################

# [CTRL-Enter] to execute a line

###### Script Lesson 22: Regression methods (continued)

par(mar=c(4,4,1,1)) # margin c(bottom, left, top, right)

# omitted variable bias
n = 100
age = runif(n, 18, 70)
experience = age + runif(n, -3, 3) # x and z correlated
cor(age, experience, method='pearson')
productivity = 2*age + 3*experience + rnorm(n)
lm(productivity~age+experience) # ok
lm(productivity~age) # omitted variable bias!

# multiple (variable) linear regression
library(MASS)
library(car)
data(Boston)
# medv is median value of owner-occupied homes in $1000s.
View(Boston)

# regression from all other variables
fit = lm(medv ~ ., data = Boston)
summary(fit)
# highly correlated variables -> multi-collinearity
d=cor(Boston)
d[abs(d) < 0.8] = NA
View(d)
# variance inflation factors (cutoff at 5)
vif(fit)
# regression from all other variables except tax
fit2 = lm(medv ~. -tax, data = Boston)
summary(fit2)
vif(fit2)

# variable selection: stepwise regression 
step = stepAIC(fit, direction="backward")
summary(step)

# penalized multiple linear regression
library(penalized) # https://cran.r-project.org/web/packages/penalized/penalized.pdf
# RIDGE: lambda2>0 and lambda1=0
# LASSO: lambda2=0 and lambda1>0
# ELASTIC NET lambda1>0 and lambda2>0
fit = penalized(medv ~. , data = Boston, lambda2=1) # Ridge
print(fit)
coef(fit)
abs(coef(fit)) < 0.01
fit = penalized(medv ~. , data = Boston, lambda1=1) # Lasso
print(fit)
coef(fit)
abs(coef(fit)) < 0.01

# Michelin dataset
setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) 
mich = read.csv('s4ds22.MichelinNY.csv')
View(mich)
attach(mich)
# cannot do linear regression inMichelin = alpha + beta rating
plot(Food, InMichelin)
fit = lm(InMichelin~Food)
abline(fit, col=2, lwd=2)
summary(fit) # small R-squared!
# can do linear regression on P(InMichelin) = alpha + beta Food
agg = tapply(InMichelin, Food, mean)
agg
rating = as.numeric(names(agg)) # Food ratings
rating
f = unname(agg) # P(InMichelin=1)
f
plot(rating, f) # S-shaped
fit = lm(f~rating)
summary(fit) # better R-squared!
abline(fit, col=2, lwd=2)
# can do better linear regression on logit(p) = alpha + beta rating
# where logit(x) = log(x/(1-x)) is the log of odds x/(1-x)
library(car) # for logit
library(boot) # for inv.logit
logit_f = car::logit(f, adjust=0.025) # adjust for 0 or 1 probabilities 
plot(f, logit_f)
curve(logit(x), col=2, lwd=2, add=T)
plot(rating, logit_f)
fit2 = lm(logit_f~rating)
summary(fit2) # even better R-squared!
abline(fit2, col=2, lwd=2)
# plot over p
plot(rating, f)
lines(rating, inv.logit(fitted(fit2)), col='red', lwd=2)
lines(rating, fitted(fit), col='blue', lwd=2)

# Generalized Linear Models
# family = error_distribution("link function")
plot(Food, InMichelin)
fit = glm(InMichelin~Food, data=mich, family=binomial("logit"))
points(Food, fitted(fit), col='red', lwd=2)
summary(fit)

fit = glm(InMichelin~Food+Decor+Service+Price,
          family=binomial("logit"), data=mich)
summary(fit)

# Elastic net logistic regression with cross-validation
library(glmnet)

# matrix of independent variables (expanding factors using dummy encoding)
x = model.matrix(InMichelin~Food+Decor+Service+Price, mich)[,-1] # remove intercept
View(x)
y = mich$InMichelin
# glmnet takes inputs: matrix and dependent variable
fit.lasso = glmnet(x, y, alpha=1, family = "binomial")
fit.lasso # include cross-validation search of lambda
coef(fit.lasso) # include cross-validation search of lambda
# cross-validation search of lambda
cv.lasso = cv.glmnet(x, y, alpha = 1, family = "binomial")
cv.lasso$lambda.min # best lambda
cv.lasso$lambda.1se # good lambda and simplest model
# Fit the final model on the training data
model = glmnet(x, y, alpha = 1, family = "binomial",
                lambda = cv.lasso$lambda.1se)
# Display regression coefficients
coef(model)
